// Copyright (C) 2009-2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function varargout = apifun_expandfromsize ( varargin )
  // Expand variables from a size.
  //
  // Calling Sequence
  //   a = apifun_expandfromsize ( 1 , a )
  //   a = apifun_expandfromsize ( 1 , a , v )
  //   a = apifun_expandfromsize ( 1 , a , m , n )
  //   [a,b] = apifun_expandfromsize ( 2 , a , b )
  //   [a,b] = apifun_expandfromsize ( 2 , a , b , v )
  //   [a,b] = apifun_expandfromsize ( 2 , a , b , m , n )
  //
  // Parameters
  //   a : a 1x1 or nxm matrix of doubles, the first parameter. 
  //   b : a 1x1 or nxm matrix of doubles, the second parameter. 
  //   v : a 1x2 or 2x1 matrix of doubles, the required size of a, b, ...
  //   v(1) : the required number of rows
  //   v(2) : the required number of columns
  //   m : a 1x1 matrix of floating point integers, the required number of rows
  //   n : a 1x1 matrix of floating point integers, the required number of columns
  //
  // Description
  // Expand the input variables from a given size.
  //
  // This function may be used to increase the flexibility of 
  // a function, which has arguments of varying dimensions.
  //
  // Examples
  // // Expand one variable
  // a = apifun_expandfromsize(1,1:6)
  // a = apifun_expandfromsize(1,0,[3 5])
  // a = apifun_expandfromsize(1,[1 2 3;4 5 6],2,3)
  // a = apifun_expandfromsize(1,0,2,3)
  // a = apifun_expandfromsize(1,[1 2 3;4 5 6],2,3)
  //
  // // Expand two variables
  // [a,b] = apifun_expandfromsize(2,1:6,(1:6)^-1)
  // [a,b] = apifun_expandfromsize(2,1:6,2)
  // [a,b] = apifun_expandfromsize(2,1,1:6)
  // [a,b] = apifun_expandfromsize(2,0,1,[3 5])
  // [a,b] = apifun_expandfromsize(2,1,[1 2 3;4 5 6],2,3)
  // [a,b] = apifun_expandfromsize(2,[1 2 3;4 5 6],0.1,2,3)
  // [a,b] = apifun_expandfromsize(2,0,1,2,3)
  // [a,b] = apifun_expandfromsize(2,[1 2 3;4 5 6],[1 2 3;4 5 6],2,3)
  //
  // // A practical use-case.
  // // The function produces normal random numbers.
  // function X = myfunction(mu,sigma,m,n)
  //     [ mu , sigma ] = apifun_expandfromsize (2,mu,sigma,m,n)
  //     u = grand(m,n,"unf",0,1)
  //     X = cdfnor("X",mu,sigma,u,1-u)
  // endfunction
  // mu = 1;
  // sigma = 2;
  // X = myfunction(mu,sigma,3,5)
  // mu = 12*ones(3,5);
  // X = myfunction(mu,sigma,3,5)
  //
  // Authors
  //   Copyright (C) 2009-2011 - DIGITEO - Michael Baudin

  [lhs,rhs]=argn()
  apifun_checkrhs ( "apifun_expandfromsize" , rhs , 2:5 )
  apifun_checklhs ( "apifun_expandfromsize" , lhs , 1:2 )
  //
  nbpar = varargin(1)
  //
  // Check npar
  apifun_checktype ( "apifun_expandfromsize" , nbpar , "nbpar" , 1 , "constant" )
  apifun_checkscalar ( "apifun_expandfromsize" , nbpar , "nbpar" , 1 )
  apifun_checkoption ( "apifun_expandfromsize" , nbpar , "nbpar" , 1 , [1 2] )
  //
  // Check number of arguments depending on nbpar
  if ( nbpar == 1 ) then
    apifun_checkrhs ( "apifun_expandfromsize" , rhs , 2 : 4 )
    a = dfxpndf_1var ( varargin(2:$) )
    apifun_checklhs ( "apifun_expandfromsize" , lhs , 1 )
    varargout(1) = a
  else
    apifun_checkrhs ( "apifun_expandfromsize" , rhs , 3 : 5 )
    [a,b] = dfxpndf_2var ( varargin(2:$) )
    apifun_checklhs ( "apifun_expandfromsize" , lhs , 2 )
    varargout(1) = a
    varargout(2) = b
  end
endfunction
function a = dfxpndf_1var ( varargin )
  // Expand 1 variable from its size
  //   a = dfxpndf_1var ( a )
  //   a = dfxpndf_1var ( a , v )
  //   a = dfxpndf_1var ( a , m , n )
  [lhs,rhs]=argn()
  select rhs
  case 1 then
    a = varargin(1)
    apifun_checktype ( "dfxpndf_1var" , a , "a" , 1 , "constant" )
  case 2 then
    a = varargin(1)
    v = varargin(2)
    apifun_checktype ( "dfxpndf_1var" , a , "a" , 1 , "constant" )
    apifun_checktype ( "dfxpndf_1var" , v , "v" , 2 , "constant" )
    apifun_checkscalar ( "dfxpndf_1var" , a , "a" , 1 )
    m = v(1)
    n = v(2)
    a = a * ones ( m , n )
  case 3 then
    a = varargin(1)
    m = varargin(2)
    n = varargin(3)
    apifun_checktype ( "dfxpndf_1var" , a , "a" , 1 , "constant" )
    apifun_checktype ( "dfxpndf_1var" , m , "m" , 2 , "constant" )
    apifun_checktype ( "dfxpndf_1var" , n , "n" , 2 , "constant" )
    if ( size(a,"*") == 1 ) then
      a = a * ones ( m , n )
    else
      if ( size(a,"*") <> 1 ) then
        apifun_checkdims ( "dfxpndf_1var" , a , "a" , 1 , [m n] )
      end
      a = apifun_expandvar ( a )
    end
  else
    errmsg = msprintf(gettext("%s: Internal error: unexpected rhs = %d."), "dfxpndf_1var" , rhs )
    error(errmsg)
  end
endfunction
function [a,b] = dfxpndf_2var ( varargin )
  // Expand 2 variables from their size
  //   [a,b] = dfxpndf_2var ( a , b )
  //   [a,b] = dfxpndf_2var ( a , b , v )
  //   [a,b] = dfxpndf_2var ( a , b , m , n )
  [lhs,rhs]=argn()
  select rhs
  case 2 then
    a = varargin(1)
    b = varargin(2)
    apifun_checktype ( "dfxpndf_2var" , a , "a" , 1 , "constant" )
    apifun_checktype ( "dfxpndf_2var" , b , "b" , 2 , "constant" )
    [ a , b ] = apifun_expandvar ( a , b )
  case 3 then
    a = varargin(1)
    b = varargin(2)
    v = varargin(3)
    apifun_checktype ( "dfxpndf_2var" , a , "a" , 1 , "constant" )
    apifun_checktype ( "dfxpndf_2var" , b , "b" , 2 , "constant" )
    apifun_checktype ( "dfxpndf_2var" , v , "v" , 3 , "constant" )
    m = v(1)
    n = v(2)
    if ( ( size(a,"*") == 1 ) & ( size(b,"*") == 1 ) ) then
      a = a * ones ( m , n )
      b = b * ones ( m , n )
    else
      if ( size(a,"*") <> 1 ) then
        apifun_checkdims ( "dfxpndf_2var" , a , "a" , 1 , [m n] )
      end
      if ( size(b,"*") <> 1 ) then
        apifun_checkdims ( "dfxpndf_2var" , b , "b" , 2 , [m n] )
      end
      [ a , b ] = apifun_expandvar ( a , b )
    end
  case 4 then
    a = varargin(1)
    b = varargin(2)
    m = varargin(3)
    n = varargin(4)
    apifun_checktype ( "dfxpndf_2var" , a , "a" , 1 , "constant" )
    apifun_checktype ( "dfxpndf_2var" , b , "b" , 2 , "constant" )
    apifun_checktype ( "dfxpndf_2var" , m , "m" , 3 , "constant" )
    apifun_checktype ( "dfxpndf_2var" , n , "n" , 4 , "constant" )
    if ( ( size(a,"*") == 1 ) & ( size(b,"*") == 1 ) ) then
      a = a * ones ( m , n )
      b = b * ones ( m , n )
    else
      if ( size(a,"*") <> 1 ) then
        apifun_checkdims ( "dfxpndf_2var" , a , "a" , 1 , [m n] )
      end
      if ( size(b,"*") <> 1 ) then
        apifun_checkdims ( "dfxpndf_2var" , b , "b" , 2 , [m n] )
      end
      [ a , b ] = apifun_expandvar ( a , b )
    end
  else
    errmsg = msprintf(gettext("%s: Internal error: unexpected rhs = %d."), "dfxpndf_2var" , rhs )
    error(errmsg)
  end
endfunction

